@@ -6,10 +6,10 @@
 #include <QUrl>
 #include <QList>
 #include <QVariant>
+#include <QScriptValue>
 
 class QNetworkAccessManager;
 class QNetworkReply;
-class ExtractorPlugin;
 
 typedef struct {
     Videos::VideoFormat format;
@@ -50,22 +50,25 @@ public slots:
     void setYouTubeFormats(QSet<int> formats);
     void setDailymotionFormats(QSet<int> formats);
     void setVimeoFormats(QSet<int> formats);
-    void cancelCurrentOperation();
+    void cancelCurrentOperation();    
+
+private:
+    void setBusy(bool isBusy, const QString &message = QString(), int numberOfOperations = 1);
+    void setCancelled(bool cancelled) { m_cancelled = cancelled; }
+    QScriptValue getYouTubeDecryptionFunction(const QUrl &playerUrl);
+    QMap<int, QUrl> getYouTubeVideoUrlMap(QString page, QScriptValue decryptionFunction = QScriptValue());
 
 private slots:
-    void parseYouTubeVideoPage();
+    void checkYouTubePageForEncryption();
+    void parseYouTubeVideoPage(QScriptValue decryptionFunction = QScriptValue(), QString page = QString());
     void parseDailymotionVideoPage();
     void parseVimeoVideoPage();
     void checkVimeoVideoRedirect();
-    void checkYouTubeVideoFormats();
+    void checkYouTubeVideoFormatsForEncryption();
+    void parseYouTubeVideoFormats(QScriptValue decryptionFunction = QScriptValue(), QString page = QString());
     void checkDailymotionVideoFormats();
     void checkVimeoVideoFormats();
-
-private:
-    void loadExtractorPlugin();
-    void setBusy(bool isBusy, const QString &message = QString(), int numberOfOperations = 1);
-    void setCancelled(bool cancelled) { m_cancelled = cancelled; }
-    QString decryptYouTubeSignature(const QString &s);
+    void addYouTubeDecryptionFunctionToCache();
 
 signals:
     void gotVideoUrl(const QUrl &url, Videos::VideoFormat videoFormat);
@@ -75,16 +78,17 @@ signals:
     void busyProgressChanged(int progress);
     void error(const QString &errorString);
     void currentOperationCancelled();
+    void youtubeDecryptionFunctionReady(QScriptValue decryptionFunction);
 
 private:
     QNetworkAccessManager *m_nam;
-    ExtractorPlugin *m_extractor;
     QList<VideoFormat> m_youtubeFormats;
     QList<VideoFormat> m_dailymotionFormats;
     QList<VideoFormat> m_vimeoFormats;
     Videos::VideoFormat m_format;
     bool m_busy;
     bool m_cancelled;
+    QString m_youtubePage;
 };
 
 #endif // URLGRABBER_H
