@@ -89,14 +89,16 @@ QString Utils::serviceString(Services::VideoService service) {
     }
 }
 
-QString Utils::reverseString(const QString &string) {
-    QString reverse;
+QString Utils::unescape(const QString &s) {
+    int unescapes = 0;
+    QByteArray us = s.toUtf8();
 
-    for (int i = string.size() - 1; i >= 0; i--) {
-        reverse.append(string.at(i));
+    while ((us.contains('%')) && (unescapes < 10)) {
+        us = QByteArray::fromPercentEncoding(us);
+        unescapes++;
     }
 
-    return reverse;
+    return QString(us);
 }
 
 void Utils::log(const QString &filePath, const QByteArray &message) {
@@ -113,6 +115,6 @@ QString Utils::versionNumberString() {
 #ifdef SYMBIAN_OS
     return QString("1.8.2");
 #else
-    return QString("1.7.8");
+    return QString("1.7.9");
 #endif
 }
