@@ -14,20 +14,28 @@
 
 using namespace QtJson;
 
+YouTube* youtubeInstance = 0;
+
+const QByteArray CLIENT_ID("446197852855.apps.googleusercontent.com");
+const QByteArray CLIENT_SECRET("YzuBGnTyRdz9cK5J3Pg5Xlte");
+const QByteArray DEVELOPER_KEY("AI39si6x9O1gQ1Z_BJqo9j2n_SdVsHu1pk2uqvoI3tVq8d6alyc1og785IPCkbVY3Q5MFuyt-IFYerMYun0MnLdQX5mo2BueSw");
+
 YouTube::YouTube(QObject *parent) :
     QObject(parent),
     m_nam(0),
-    m_clientId("446197852855.apps.googleusercontent.com"),
-    m_clientSecret("YzuBGnTyRdz9cK5J3Pg5Xlte"),
-    m_key("AI39si6x9O1gQ1Z_BJqo9j2n_SdVsHu1pk2uqvoI3tVq8d6alyc1og785IPCkbVY3Q5MFuyt-IFYerMYun0MnLdQX5mo2BueSw"),
     m_safeSearch(false),
     m_actionsProcessed(0),
     m_playlistCache(new QList< QSharedPointer<PlaylistItem> >),
     m_subscriptionCache(new QList< QSharedPointer<UserItem> >),
     m_playlistCacheLoaded(false),
     m_subscriptionCacheLoaded(false),
-    m_busy(false)
+    m_busy(false),
+    m_cancelled(false)
 {
+    if (!youtubeInstance) {
+        youtubeInstance = this;
+    }
+
     m_queryOrders[Queries::Relevance] = "relevance";
     m_queryOrders[Queries::Date] = "published";
     m_queryOrders[Queries::Views] = "viewCount";
@@ -47,14 +55,19 @@ YouTube::~YouTube() {
     this->clearCache();
 }
 
+YouTube* YouTube::instance() {
+    return youtubeInstance;
+}
+
 void YouTube::setBusy(bool isBusy, const QString &message, int numberOfOperations) {
     if (isBusy != this->busy()) {
         m_busy = isBusy;
 
         if (isBusy) {
+            this->setCancelled(false);
             emit busy(message, numberOfOperations);
         }
-        else {
+        else if (!this->cancelled()) {
             emit busyProgressChanged(numberOfOperations);
         }
 
@@ -62,22 +75,34 @@ void YouTube::setBusy(bool isBusy, const QString &message, int numberOfOperation
     }
 }
 
-QNetworkReply* YouTube::createReply(QString feed, int offset) {
-    QUrl url(feed);
+void YouTube::cancelCurrentOperation() {
+    this->setCancelled(true);
+    this->setBusy(false);
+    this->disconnect(this, SIGNAL(postSuccessful(QString)), this, 0);
+    this->disconnect(this, SIGNAL(postFailed(QString)), this, 0);
+    this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, 0);
+#ifdef QML_USER_INTERFACE
+    this->disconnect(this, SIGNAL(gotVideo(VideoItem*)), 0, 0);
+#else
+    this->disconnect(this, SIGNAL(gotVideo(QSharedPointer<VideoItem>)), 0, 0);
+#endif
+    emit currentOperationCancelled();
+}
 
+QNetworkReply* YouTube::createReply(QUrl feed, int offset) {
     if (this->userSignedIn()) {
-        url.addQueryItem("access_token", this->accessToken());
+        feed.addQueryItem("access_token", this->accessToken());
     }
 
     if (offset) {
-        url.addQueryItem("start-index", QString::number(offset));
+        feed.addQueryItem("start-index", QString::number(offset));
     }
 
-    qDebug() << url;
+    qDebug() << feed;
 
-    QNetworkRequest request(url);
+    QNetworkRequest request(feed);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    request.setRawHeader("X-Gdata-Key", "key=" + this->developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
 
     return this->networkAccessManager()->get(request);
 }
@@ -85,7 +110,8 @@ QNetworkReply* YouTube::createReply(QString feed, int offset) {
 QNetworkReply* YouTube::createSearchReply(int queryType, const QString &query, int offset, int order, int time, int duration, const QString &language) {
     QUrl url;
 
-    if (queryType == Queries::Videos) {
+    switch (queryType) {
+    case Queries::Videos:
         url.setUrl(YOUTUBE_VIDEOS_BASE_URL);
         url.addQueryItem("fields", YOUTUBE_VIDEO_FIELDS);
         url.addQueryItem("orderby", m_queryOrders.value(order, "relevance"));
@@ -94,17 +120,23 @@ QNetworkReply* YouTube::createSearchReply(int queryType, const QString &query, i
         if (duration != Queries::Any) {
             url.addQueryItem("duration", m_durationFilters.value(duration, "medium"));
         }
-        if (language != "all") {
+        if ((!language.isEmpty()) && (language != "all")) {
             url.addQueryItem("lr", language);
         }
-    }
-    else if (queryType == Queries::Playlists) {
+        break;
+    case Queries::Playlists:
         url.setUrl(YOUTUBE_PLAYLISTS_SEARCH_BASE_URL);
         url.addQueryItem("fields", YOUTUBE_PLAYLIST_FIELDS);
-    }
-    else if (queryType == Queries::Users) {
+        url.addQueryItem("orderby", "relevance");
+        break;
+    case Queries::Users:
         url.setUrl(YOUTUBE_CHANNELS_BASE_URL);
         url.addQueryItem("fields", YOUTUBE_USER_FIELDS);
+        url.addQueryItem("orderby", "relevance");
+        break;
+    default:
+        qWarning() << "YouTube::createSearchReply(): No/invalid query type.";
+        break;
     }
 
     url.addQueryItem("q", QString("\"%1\"|%2").arg(query).arg(query.simplified().replace(' ', '+')));
@@ -124,9 +156,39 @@ QNetworkReply* YouTube::createSearchReply(int queryType, const QString &query, i
     return this->networkAccessManager()->get(request);
 }
 
+QNetworkReply* YouTube::createUploadReply(const QVariantMap &metadata) {
+    QUrl url("http://uploads.gdata.youtube.com/resumable/feeds/api/users/default/uploads");
+    QByteArray xml("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" \
+                   "<entry xmlns=\"http://www.w3.org/2005/Atom\"\n" \
+                   "xmlns:media=\"http://search.yahoo.com/mrss/\"\n" \
+                   "xmlns:yt=\"http://gdata.youtube.com/schemas/2007\">\n" \
+                   "<media:group>\n" \
+                   "<media:title>" + metadata.value("title").toString().toUtf8() + "</media:title>\n" \
+                   "<media:description>\n" + metadata.value("description").toString().toUtf8() + "\n\n" + metadata.value("uploadAttribute").toString().toUtf8() + "\n</media:description>\n" \
+                   "<media:category scheme=\"http://gdata.youtube.com/schemas/2007/categories.cat\">\n" + metadata.value("category").toString().toUtf8() + "\n</media:category>\n" \
+                   "<media:keywords>" + metadata.value("tags").toString().toUtf8() + "</media:keywords>\n" \
+                   "</media:group>\n" \
+                   "</entry>");
+
+    if (metadata.value("isPrivate").toBool()) {
+        xml.insert(xml.indexOf("</media:group>"), "<yt:private/>\n");
+    }
+
+    QNetworkRequest request(url);
+    request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
+    request.setRawHeader("Host", "uploads.gdata.youtube.com");
+    request.setHeader(QNetworkRequest::ContentTypeHeader, "application/atom+xml; charset=UTF-8");
+    request.setHeader(QNetworkRequest::ContentLengthHeader, xml.length());
+    request.setRawHeader("Authorization", "Bearer " + this->accessToken().toUtf8());
+    request.setRawHeader("GData-Version", "2");
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
+    request.setRawHeader("Slug", metadata.value("filePath").toString().section('/', -1).toUtf8());
+    return this->networkAccessManager()->post(request, xml);
+}
+
 QUrl YouTube::authUrl() const {
     QUrl url("https://accounts.google.com/o/oauth2/auth");
-    url.addQueryItem("client_id", m_clientId);
+    url.addQueryItem("client_id", CLIENT_ID);
     url.addQueryItem("redirect_uri", "urn:ietf:wg:oauth:2.0:oob");
     url.addQueryItem("response_type", "code");
     url.addQueryItem("scope", "https://gdata.youtube.com");
@@ -142,8 +204,8 @@ void YouTube::signIn(const QString &displayName, const QString &code) {
     QUrl url("https://accounts.google.com/o/oauth2/token");
 
     QByteArray body("code=" + code.toUtf8() +
-                    "&client_id=" + m_clientId.toUtf8() +
-                    "&client_secret=" + m_clientSecret.toUtf8() +
+                    "&client_id=" + CLIENT_ID +
+                    "&client_secret=" + CLIENT_SECRET +
                     "&redirect_uri=urn:ietf:wg:oauth:2.0:oob&grant_type=authorization_code");
 
     QNetworkRequest request(url);
@@ -151,11 +213,12 @@ void YouTube::signIn(const QString &displayName, const QString &code) {
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/x-www-form-urlencoded");
     QNetworkReply *reply = this->networkAccessManager()->post(request, body);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkIfSignedIn()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void YouTube::checkIfSignedIn() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -182,13 +245,15 @@ void YouTube::checkIfSignedIn() {
             emit signedIn(this->username(), token, refresh);
         }
     }
+
+    reply->deleteLater();
 }
 
 void YouTube::refreshAccessToken() {
     QUrl url("https://accounts.google.com/o/oauth2/token");
 
-    QByteArray body("client_id=" + m_clientId.toUtf8() +
-                    "&client_secret=" + m_clientSecret.toUtf8() +
+    QByteArray body("client_id=" + CLIENT_ID +
+                    "&client_secret=" + CLIENT_SECRET +
                     "&refresh_token=" + refreshToken().toUtf8() +
                     "&grant_type=refresh_token");
 
@@ -200,7 +265,7 @@ void YouTube::refreshAccessToken() {
 }
 
 void YouTube::checkTokenRefresh() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -255,8 +320,6 @@ void YouTube::linkGoogleAccount(QString username) {
 }
 
 void YouTube::onGoogleAccountLinked(const QString &response) {
-    m_linkUsername = "";
-
     QDomDocument doc;
     doc.setContent(response);
     QDomNode entry = doc.firstChildElement("entry");
@@ -269,20 +332,19 @@ void YouTube::onGoogleAccountLinked(const QString &response) {
 }
 
 void YouTube::checkUsernameAvailability(QString username) {
-    m_linkUsername = username;
     QUrl url("https://gdata.youtube.com/feeds/api/suggest/username");
     url.addQueryItem("v", "2.1");
     url.addQueryItem("fields", "entry/title");
-    url.addQueryItem("hint", m_linkUsername);
+    url.addQueryItem("hint", username);
     QNetworkRequest request(url);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
-    request.setRawHeader("X-Gdata-Key", "key=" + developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkSuggestedUsernames()));
 }
 
 void YouTube::checkSuggestedUsernames() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -301,9 +363,9 @@ void YouTube::checkSuggestedUsernames() {
     if (usernames.isEmpty()) {
         emit usernameUnavailable();
     }
-    else if (usernames.contains(m_linkUsername)) {
+    else if (usernames.contains(reply->request().url().queryItemValue("hint"))) {
         emit usernameAvailable();
-        this->linkGoogleAccount(m_linkUsername);
+        this->linkGoogleAccount(reply->request().url().queryItemValue("hint"));
     }
     else {
         emit gotSuggestedUsernames(usernames);
@@ -348,9 +410,10 @@ void YouTube::postRequest(const QUrl &url, const QByteArray &xml) {
     request.setHeader(QNetworkRequest::ContentLengthHeader, xml.length());
     request.setRawHeader("Authorization", "Bearer " + this->accessToken().toUtf8());
     request.setRawHeader("GData-Version", "2.1");
-    request.setRawHeader("X-Gdata-Key", "key=" + this->developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
     QNetworkReply* reply = this->networkAccessManager()->post(request, xml);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void YouTube::putRequest(const QUrl &url, const QByteArray &xml) {
@@ -363,9 +426,10 @@ void YouTube::putRequest(const QUrl &url, const QByteArray &xml) {
     request.setHeader(QNetworkRequest::ContentLengthHeader, xml.length());
     request.setRawHeader("Authorization", "Bearer " + this->accessToken().toUtf8());
     request.setRawHeader("GData-Version", "2.1");
-    request.setRawHeader("X-Gdata-Key", "key=" + this->developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
     QNetworkReply* reply = this->networkAccessManager()->put(request, xml);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void YouTube::deleteRequest(const QUrl &url) {
@@ -376,13 +440,14 @@ void YouTube::deleteRequest(const QUrl &url) {
     request.setHeader(QNetworkRequest::ContentTypeHeader, "application/atom+xml");
     request.setRawHeader("Authorization", "Bearer " + this->accessToken().toUtf8());
     request.setRawHeader("GData-Version", "2.1");
-    request.setRawHeader("X-Gdata-Key", "key=" + this->developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
     QNetworkReply* reply = this->networkAccessManager()->deleteResource(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(postFinished()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void YouTube::postFinished() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -441,7 +506,7 @@ void YouTube::getPlaylists(int offset) {
 }
 
 void YouTube::addPlaylists() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -466,14 +531,11 @@ void YouTube::addPlaylists() {
             m_playlistCache->append(QSharedPointer<PlaylistItem>(playlist));
         }
 
-        this->setPlaylistsLoaded(true);
-
-        int totalResults = doc.namedItem("feed").firstChildElement("openSearch:totalResults").text().toInt();
-
-        if ((totalResults > m_playlistCache->size()) && (!entries.isEmpty())) {
+        if ((!doc.namedItem("feed").namedItem("link").isNull()) && (!entries.isEmpty())) {
             this->getPlaylists(m_playlistCache->size() + 1);
         }
         else {
+            this->setPlaylistsLoaded(true);
             emit allPlaylistsLoaded();
         }
 
@@ -550,7 +612,7 @@ void YouTube::getSubscriptions(int offset) {
 }
 
 void YouTube::addSubscriptions() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -572,18 +634,14 @@ void YouTube::addSubscriptions() {
         for (int i = 0; i < entries.size(); i++) {
             UserItem *user = new UserItem;
             user->loadYouTubeUser(entries.at(i), true, true);
-            this->connect(this, SIGNAL(subscriptionChanged(QString,bool,QString)), user, SLOT(onSubscriptionChanged(QString,bool,QString)));
             m_subscriptionCache->append(QSharedPointer<UserItem>(user));
         }
 
-        this->setSubscriptionsLoaded(true);
-
-        int totalResults = doc.namedItem("feed").firstChildElement("openSearch:totalResults").text().toInt();
-
-        if ((totalResults > m_subscriptionCache->size()) && (!entries.isEmpty())) {
+        if ((!doc.namedItem("feed").namedItem("link").isNull()) && (!entries.isEmpty())) {
             this->getSubscriptions(m_subscriptionCache->size() + 1);
         }
         else {
+            this->setSubscriptionsLoaded(true);
             emit allSubscriptionsLoaded();
         }
 
@@ -595,18 +653,9 @@ void YouTube::addSubscriptions() {
 }
 
 void YouTube::addNewSubscriptionToCache(QSharedPointer<UserItem> user) {
-    int i = 0;
-    bool found = false;
-
-    while ((!found) && (i < m_subscriptionCache->size())) {
-        found = (m_subscriptionCache->at(i).data()->username() > user.data()->username());
-        i++;
-    }
-
-    m_subscriptionCache->insert(i - 1, user);
-    emit subscriptionAddedToCache(i - 1);
+    m_subscriptionCache->insert(0, user);
+    emit subscriptionAddedToCache(0);
     emit subscriptionChanged(user.data()->id(), true, user.data()->subscriptionId());
-    this->connect(this, SIGNAL(subscriptionChanged(QString,bool,QString)), user.data(), SLOT(onSubscriptionChanged(QString,bool,QString)));
 }
 
 QSharedPointer<UserItem> YouTube::removeSubscriptionFromCache(const QString &id) {
@@ -623,7 +672,7 @@ QSharedPointer<UserItem> YouTube::removeSubscriptionFromCache(const QString &id)
     if (removed) {
         user = m_subscriptionCache->takeAt(i - 1);
         emit subscriptionRemovedFromCache(i - 1);
-        emit subscriptionChanged(id, false);
+        emit subscriptionChanged(id, false, QString());
     }
 
     return user;
@@ -663,17 +712,22 @@ void YouTube::getFullVideo(QString id) {
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkFullVideo()));
+    this->connect(this, SIGNAL(currentOperationCancelled()), reply, SLOT(deleteLater()));
 }
 
 void YouTube::checkFullVideo() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
         return;
     }
 
+    if (this->cancelled()) {
+        return;
+    }
+
     int statusCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
     if (statusCode == 200) {
@@ -681,9 +735,6 @@ void YouTube::checkFullVideo() {
         doc.setContent(reply->readAll());
         VideoItem *video = new VideoItem;
         video->loadYouTubeVideo(doc.namedItem("entry"));
-        this->connect(this, SIGNAL(favouriteChanged(QString,bool,QString)), video, SLOT(onFavouriteChanged(QString,bool,QString)));
-        this->connect(this, SIGNAL(videoLiked(QString)), video, SLOT(onVideoLiked(QString)));
-        this->connect(this, SIGNAL(videoDisliked(QString)), video, SLOT(onVideoDisliked(QString)));
 #ifdef QML_USER_INTERFACE
         QDeclarativeEngine::setObjectOwnership(video, QDeclarativeEngine::JavaScriptOwnership);
         emit gotVideo(video);
@@ -693,7 +744,11 @@ void YouTube::checkFullVideo() {
     }
     else {
         emit error(tr("Video could not be retrieved"));
+#ifdef QML_USER_INTERFACE
+        this->disconnect(this, SIGNAL(gotVideo(VideoItem*)), 0, 0);
+#else
         this->disconnect(this, SIGNAL(gotVideo(QSharedPointer<VideoItem>)), 0, 0);
+#endif
     }
 
     reply->deleteLater();
@@ -710,7 +765,7 @@ void YouTube::getVideoMetadata(const QString &id) {
 }
 
 void YouTube::checkVideoMetadata() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -771,7 +826,7 @@ void YouTube::checkVideoMetadata() {
 }
 
 void YouTube::getFullComment(const QString &videoId, const QString &commentId) {
-    QUrl url(YOUTUBE_VIDEOS_BASE_URL + QString("/") + videoId + QString("/comments/") + commentId);
+    QUrl url(YOUTUBE_VIDEOS_BASE_URL + QString("/%1/comments/%2").arg(videoId).arg(commentId));
     url.addQueryItem("v", "2.1");
     url.addQueryItem("fields", YOUTUBE_SINGLE_COMMENT_FIELDS);
     QNetworkRequest request(url);
@@ -781,7 +836,7 @@ void YouTube::getFullComment(const QString &videoId, const QString &commentId) {
 }
 
 void YouTube::checkFullComment() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -816,7 +871,7 @@ void YouTube::getCurrentUserProfile() {
 }
 
 void YouTube::checkCurrentUserProfile() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -864,7 +919,7 @@ void YouTube::getUserProfile(const QString &id) {
 }
 
 void YouTube::checkUserProfile() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -878,7 +933,6 @@ void YouTube::checkUserProfile() {
         doc.setContent(reply->readAll());
         UserItem *user = new UserItem;
         user->loadYouTubeUser(doc.namedItem("entry"));
-        this->connect(this, SIGNAL(subscriptionChanged(QString,bool,QString)), user, SLOT(onSubscriptionChanged(QString,bool,QString)));
 
         if (this->subscriptionsLoaded()) {
             this->subscribedToChannel(user->id());
@@ -920,7 +974,7 @@ void YouTube::updateVideoMetadata() {
                    "<media:keywords>" + m_metadataAction.value("tags").toString().toUtf8() + "</media:keywords>\n" \
                    "</media:group>\n" \
                    "<yt:accessControl action=\"comment\" permission=\"" + m_metadataAction.value("commentsPermission").toString().toUtf8() + "\"/>\n" \
-                   "<yt:accessControl action=\"commentVote\" permission=\"" + m_metadataAction.value("commentVotingPermission").toString().toUtf8() + "\"/>\n" \
+                   "<yt:accessControl action=\"commentVote\" permission=\"" + m_metadataAction.value("commentVotePermission").toString().toUtf8() + "\"/>\n" \
                    "<yt:accessControl action=\"rate\" permission=\"" + m_metadataAction.value("ratingsPermission").toString().toUtf8() + "\"/>\n" \
                    "<yt:accessControl action=\"syndicate\" permission=\"" + m_metadataAction.value("syndicationPermission").toString().toUtf8() + "\"/>\n" \
                    "<yt:accessControl action=\"list\" permission=\"" + m_metadataAction.value("listingsPermission").toString().toUtf8() + "\"/>\n" \
@@ -971,22 +1025,24 @@ void YouTube::deleteFromUploads() {
 }
 
 void YouTube::onVideoDeleted() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit deletedFromUploads(m_actionIdList.takeFirst());
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromUploads();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from uploads"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onVideoDeleted()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromUploads()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromUploads();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from uploads"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onVideoDeleted()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromUploads()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1022,22 +1078,24 @@ void YouTube::onAddedToFavourites(const QString &response) {
     doc.setContent(response);
     QDomNode entry = doc.firstChildElement("entry");
 
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit favouriteChanged(m_actionIdList.takeFirst(), true, entry.firstChildElement("id").text().section(':', -1));
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->addToFavourites();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) added to favourites"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToFavourites(QString)));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToFavourites()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->addToFavourites();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) added to favourites"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToFavourites(QString)));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToFavourites()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1064,22 +1122,24 @@ void YouTube::deleteFromFavourites() {
 }
 
 void YouTube::onDeletedFromFavourites() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
-        emit favouriteChanged(m_actionIdList.takeFirst(), false);
+        emit favouriteChanged(m_actionIdList.takeFirst(), false, QString());
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromFavourites();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from favourites"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromFavourites()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromFavourites()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromFavourites();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from favourites"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromFavourites()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromFavourites()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1114,9 +1174,6 @@ void YouTube::addToPlaylist() {
 }
 
 void YouTube::onAddedToPlaylist(const QString &response) {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (this->playlistsLoaded()) {
         QDomDocument doc;
         doc.setContent(response);
@@ -1128,16 +1185,21 @@ void YouTube::onAddedToPlaylist(const QString &response) {
     if (!m_actionIdList.isEmpty()) {
         emit addedToPlaylist(m_actionIdList.takeFirst(), m_actionId);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->addToPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) added to playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToPlaylist(QString)));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->addToPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) added to playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToPlaylist(QString)));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1165,25 +1227,27 @@ void YouTube::deleteFromPlaylist() {
 }
 
 void YouTube::onDeletedFromPlaylist() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (this->playlistsLoaded()) {
         this->updatePlaylistVideoCount(m_actionId, -1);
     }
     if (!m_actionIdList.isEmpty()) {
         emit deletedFromPlaylist(m_actionIdList.takeFirst(), m_actionId);
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromPlaylist()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromPlaylist()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1218,22 +1282,24 @@ void YouTube::addToWatchLaterPlaylist() {
 void YouTube::onAddedToWatchLaterPlaylist(const QString &response) {
     Q_UNUSED(response)
 
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
     if (!m_actionIdList.isEmpty()) {
         emit addedToWatchLaterPlaylist(m_actionIdList.takeFirst());
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->addToWatchLaterPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) added to 'Watch Later' playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToWatchLaterPlaylist(QString)));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToWatchLaterPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->addToWatchLaterPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) added to 'Watch Later' playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onAddedToWatchLaterPlaylist(QString)));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(addToWatchLaterPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1259,23 +1325,25 @@ void YouTube::deleteFromWatchLaterPlaylist() {
     }
 }
 
-void YouTube::onDeletedFromWatchLaterPlaylist() {
-    m_actionsProcessed++;
-    emit busyProgressChanged(m_actionsProcessed);
-
+void YouTube::onDeletedFromWatchLaterPlaylist() {    
     if (!m_actionIdList.isEmpty()) {
         emit deletedFromWatchLaterPlaylist(m_actionIdList.takeFirst());
     }
-    if (!m_actionIdList.isEmpty()) {
-        this->deleteFromWatchLaterPlaylist();
-    }
-    else {
-        this->setBusy(false);
-        emit alert(tr("Video(s) deleted from 'Watch Later' playlist"));
+    if (!this->cancelled()) {
+        m_actionsProcessed++;
+        emit busyProgressChanged(m_actionsProcessed);
 
-        this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromWatchLaterPlaylist()));
-        this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromWatchLaterPlaylist()));
-        this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        if (!m_actionIdList.isEmpty()) {
+            this->deleteFromWatchLaterPlaylist();
+        }
+        else {
+            this->setBusy(false);
+            emit alert(tr("Video(s) deleted from 'Watch Later' playlist"));
+
+            this->disconnect(this, SIGNAL(postSuccessful(QString)), this, SLOT(onDeletedFromWatchLaterPlaylist()));
+            this->disconnect(this, SIGNAL(accessTokenRefreshed(QString)), this, SLOT(deleteFromWatchLaterPlaylist()));
+            this->disconnect(this, SIGNAL(postFailed(QString)), this, SLOT(onVideoActionError(QString)));
+        }
     }
 }
 
@@ -1343,14 +1411,14 @@ void YouTube::getPlaylistForCache(const QString &id) {
     url.addQueryItem("fields", YOUTUBE_SINGLE_PLAYLIST_FIELDS);
     url.addQueryItem("access_token", this->accessToken());
     QNetworkRequest request(url);
-    request.setRawHeader("X-Gdata-Key", "key=" + this->developerKey().toUtf8());
+    request.setRawHeader("X-Gdata-Key", "key=" + DEVELOPER_KEY);
     request.setRawHeader("User-Agent", QString("cuteTube/%1 (Qt)").arg(Utils::versionNumberString()).toUtf8());
     QNetworkReply *reply = this->networkAccessManager()->get(request);
     this->connect(reply, SIGNAL(finished()), this, SLOT(checkCachePlaylist()));
 }
 
 void YouTube::checkCachePlaylist() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1430,6 +1498,9 @@ void YouTube::onSubscribed(const QString &response) {
         user->loadYouTubeUser(entry, true, true);
         this->addNewSubscriptionToCache(QSharedPointer<UserItem>(user));
     }
+    else {
+        emit subscriptionChanged(m_actionId, true, entry.firstChildElement("id").text().section(':', -1));
+    }
 
     emit alert(tr("You have subscribed to '%1'").arg(entry.firstChildElement("yt:username").attribute("display")));
 
@@ -1459,10 +1530,12 @@ void YouTube::onUnsubscribed() {
             emit alert(tr("You have unsubscribed to '%1'").arg(user.data()->username()));
         }
         else {
+            emit subscriptionChanged(m_actionId, false, QString());
             emit alert(tr("You have unsubscribed to this channel"));
         }
     }
     else {
+        emit subscriptionChanged(m_actionId, false, QString());
         emit alert(tr("You have unsubscribed to this channel"));
     }
 
@@ -1585,7 +1658,7 @@ void YouTube::onCommentAdded(const QString &response) {
 }
 
 void YouTube::getAddedComment(const QString &videoId, const QString &commentId) {
-    QUrl url(QString("%1/%2/comments/%3").arg(YOUTUBE_VIDEOS_BASE_URL).arg(videoId).arg(commentId));
+    QUrl url(QString("%1/%2/comments/%3").arg(QString(YOUTUBE_VIDEOS_BASE_URL)).arg(videoId).arg(commentId));
     url.addQueryItem("v", "2.1");
     url.addQueryItem("fields", YOUTUBE_SINGLE_COMMENT_FIELDS);
     QNetworkRequest request(url);
@@ -1595,7 +1668,7 @@ void YouTube::getAddedComment(const QString &videoId, const QString &commentId)
 }
 
 void YouTube::checkAddedComment() {
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1649,7 +1722,7 @@ void YouTube::onCommentActionError(const QString &errorString) {
 }
 
 void YouTube::getVideosFromIds(QStringList ids) {
-    this->setBusy(true);
+    this->setBusy(true, QString(), 0);
     QUrl url("https://gdata.youtube.com/feeds/api/videos/batch");
     url.addQueryItem("v", "2.1");
 
@@ -1678,7 +1751,7 @@ void YouTube::getVideosFromIds(QStringList ids) {
 
 void YouTube::checkFullVideos() {
     this->setBusy(false);
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(sender());
+    QNetworkReply* reply = qobject_cast<QNetworkReply*>(this->sender());
 
     if (!reply) {
         emit error(tr("Network error"));
@@ -1693,12 +1766,11 @@ void YouTube::checkFullVideos() {
         QList< QSharedPointer<VideoItem> > videos;
 
         for (int i = 0; i < entries.size(); i++) {
-            VideoItem *video = new VideoItem;
-            video->loadYouTubeVideo(entries.at(i));
-            this->connect(this, SIGNAL(favouriteChanged(QString,bool,QString)), video, SLOT(onFavouriteChanged(QString,bool,QString)));
-            this->connect(this, SIGNAL(videoLiked(QString)), video, SLOT(onVideoLiked(QString)));
-            this->connect(this, SIGNAL(videoDisliked(QString)), video, SLOT(onVideoDisliked(QString)));
-            videos.append(QSharedPointer<VideoItem>(video));
+            if (entries.at(i).firstChildElement("batch:status").attribute("code").toInt() == 200) {
+                VideoItem *video = new VideoItem;
+                video->loadYouTubeVideo(entries.at(i));
+                videos.append(QSharedPointer<VideoItem>(video));
+            }
         }
 
         emit gotVideosFromIds(videos);
