@@ -3,16 +3,17 @@
 #include "transferprioritydialog.h"
 #include "transferdetailsdialog.h"
 #include "listview.h"
-#include "../base/transferlistmodel.h"
+#include "transferlistmodel.h"
 #include <QLabel>
 #include <QMenu>
 #include <QAction>
+#include <QMaemo5Style>
 
 TransferListWindow::TransferListWindow(QWidget *parent) :
     StackedWindow(parent),
     m_model(new TransferListModel(this)),
     m_listView(new ListView(this)),
-    m_noResultsLabel(new QLabel(QString("<p style='color:#4d4d4d;font-size:40px;'><center><b>%1</b></center></p>").arg(tr("No transfers queued")), this)),
+    m_noResultsLabel(new QLabel(QString("<p style='color:%1;font-size:40px;'><center><b>%2</b></center></p>").arg(QMaemo5Style::standardColor("SecondaryTextColor").name()).arg(tr("No transfers queued")), this)),
     m_contextMenu(new QMenu(this)),
     m_saveAsAudioAction(m_contextMenu->addAction(tr("Save as audio"), this, SLOT(setTransferSaveAsAudio()))),
     m_priorityAction(m_contextMenu->addAction(tr("Set priority"), this, SLOT(showPriorityDialog()))),
