@@ -3,19 +3,20 @@
 
 #include "abstracttab.h"
 
-class Session;
 class VimeoUserListModel;
 class ListView;
 class QLabel;
 class QGridLayout;
 class QModelIndex;
+class QUrl;
 
 class VimeoUserListTab : public AbstractTab
 {
     Q_OBJECT
+
 public:
-    explicit VimeoUserListTab(Session *session, const QString &userId, QWidget *parent = 0);
-    void setFeed(const QString &feed);
+    explicit VimeoUserListTab(const QString &userId, QWidget *parent = 0);
+    void setFeed(const QUrl &feed);
 
 private:
     void setLandscapeLayout();
@@ -28,7 +29,6 @@ private slots:
     void showUserChannel(const QString &userId);
 
 private:
-    Session *m_session;
     QString m_userId;
     VimeoUserListModel *m_model;
     ListView *m_listView;
