@@ -21,6 +21,7 @@
 #include <QPushButton>
 #include <QScrollArea>
 #include <QGridLayout>
+#include <QMaemo5Style>
 
 YouTubeVideoDescriptionTab::YouTubeVideoDescriptionTab(QSharedPointer<VideoItem> video, QWidget *parent) :
     AbstractTab(parent),
@@ -41,8 +42,8 @@ YouTubeVideoDescriptionTab::YouTubeVideoDescriptionTab(QSharedPointer<VideoItem>
     m_dateLabel(new QLabel(QString("%1 - %2").arg(m_video.data()->date()).arg(m_video.data()->artist()), this)),
     m_likesLabel(new QLabel(QString::number(m_video.data()->likes()), this)),
     m_dislikesLabel(new QLabel(QString::number(m_video.data()->dislikes()), this)),
-    m_viewsLabel(new QLabel(QString("%1   <font color='#4d4d4d'>%2</font>").arg(tr("Views")).arg(m_video.data()->viewCount()), this)),
-    m_descriptionLabel(new QLabel(QString("Description<br><font color='#4d4d4d'>%1</font>").arg(m_video.data()->description().isEmpty() ? tr("No description") : m_video.data()->description()), this)),
+    m_viewsLabel(new QLabel(QString("%1   <font color='%2'>%3</font>").arg(tr("Views")).arg(QMaemo5Style::standardColor("SecondaryTextColor").name()).arg(m_video.data()->viewCount()), this)),
+    m_descriptionLabel(new QLabel(QString("%1<br><font color='%2'>%3</font>").arg(tr("Description")).arg(QMaemo5Style::standardColor("SecondaryTextColor").name()).arg(m_video.data()->description().isEmpty() ? tr("No description") : m_video.data()->description()), this)),
     m_scrollArea(new QScrollArea(this)),
     m_grid(new QGridLayout(this))
 {
